//+------------------------------------------------------------------+
//|                                          adaptive price zone.mq4 |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_chart_window
#property indicator_buffers  3
#property indicator_color1   Lime
#property indicator_color2   Red
#property indicator_color3   White
#property indicator_style1   STYLE_DOT
#property indicator_style2   STYLE_DOT
#property indicator_style3   STYLE_DOT

//
//
//
//
//

extern int    emaPeriod         = 14;
extern int    emaPrice          = PRICE_CLOSE;
extern double bandPercent       = 2;
extern bool   alertsOn          = False;
extern bool   alertsOnCurrent   = true;
extern bool   alertsMessage     = true;
extern bool   alertsSound       = false;
extern bool   alertsEmail       = false;
extern bool   alertsPush        = true;
extern bool ShowArrows = true;
extern color ArrowsUpColor = Green,
             ArrowsDnColor = Red;

//
//
//
//
//

double upBuffer[];
double dnBuffer[];
double mdBuffer[];
double trend[];
double wrkBuffer[][4];
double alpha;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(4);
   SetIndexBuffer(0,upBuffer);
   SetIndexBuffer(1,dnBuffer);
   SetIndexBuffer(2,mdBuffer);
   SetIndexBuffer(3,trend);
   emaPeriod = MathMax(emaPeriod,1);
       alpha = 2.0 /(1.0+MathSqrt(emaPeriod));
   
   return(0);
}
int deinit() {
   if (ShowArrows) {
      for (int i=ObjectsTotal()-1;i>=0;i--) {
         if (StringFind(ObjectName(i),"Trade:")>-1) ObjectDelete(ObjectName(i));
         }
      }
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

#define emaa 0
#define emab 1
#define rnga 2
#define rngb 3

//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,r,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         if (ArrayRange(wrkBuffer,0) != Bars) ArrayResize(wrkBuffer,Bars);
         
   //
   //
   //
   //
   //
            
   for(i=limit, r=Bars-i-1; i>=0; i--,r++)
   {
      double price = iMA(NULL,0,1,0,MODE_SMA,emaPrice,i);
      double range = iMA(NULL,0,1,0,MODE_SMA,PRICE_HIGH,i)-iMA(NULL,0,1,0,MODE_SMA,PRICE_LOW,i);
         if (i==Bars-1)
         {
            upBuffer[i]        = price;
            dnBuffer[i]        = price;
            wrkBuffer[r][emaa] = price;
            wrkBuffer[r][emab] = price;
            wrkBuffer[r][rnga] = range;
            wrkBuffer[r][rngb] = range;
            continue;
         }
         wrkBuffer[r][emaa] = wrkBuffer[r-1][emaa]+alpha*(price             -wrkBuffer[r-1][emaa]);
         wrkBuffer[r][emab] = wrkBuffer[r-1][emab]+alpha*(wrkBuffer[r][emaa]-wrkBuffer[r-1][emab]);
         wrkBuffer[r][rnga] = wrkBuffer[r-1][rnga]+alpha*(range             -wrkBuffer[r-1][rnga]);
         wrkBuffer[r][rngb] = wrkBuffer[r-1][rngb]+alpha*(wrkBuffer[r][rnga]-wrkBuffer[r-1][rngb]);

         //
         //
         //
         //
         //
         
         upBuffer[i] = wrkBuffer[r][emab]+bandPercent*wrkBuffer[r][rngb];
         dnBuffer[i] = wrkBuffer[r][emab]-bandPercent*wrkBuffer[r][rngb];
         mdBuffer[i] = wrkBuffer[r][emab];
         trend[i]    = trend[i+1];
            if (High[i]>upBuffer[i]) {
               trend[i] =  1;
               if (ShowArrows) DrawArrow(i,ArrowsDnColor,234,true);
               }
            if (Low[i] <dnBuffer[i]) {
               trend[i] = -1;
               if (ShowArrows) DrawArrow(i,ArrowsUpColor,233,false);
               }      
   }
   manageAlerts();
   return(0);
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts() {
   int whichBar;
   if (alertsOnCurrent)
      whichBar = 0;
   else whichBar = 1;
   if (trend[whichBar] != trend[whichBar+1]) {
      if (trend[whichBar] ==  1) {
         if (alertsOn) doAlert(whichBar,"high crossed upper band");
         }
      if (trend[whichBar] == -1) {
         if (alertsOn) doAlert(whichBar,"low crossed lower band");
         }
      }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," ",EnumToString((ENUM_TIMEFRAMES)Period())," adaptive price zone ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol()," adaptive price zone "),message);
          if (alertsPush)    SendNotification(message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}
void DrawArrow(int i,color theColor,int theCode,bool up) {
   string name = "Trade:"+Time[i];
   double gap  = 3.0*iATR(NULL,0,20,i)/4.0;   
   ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
      ObjectSet(name,OBJPROP_ARROWCODE,theCode);
      ObjectSet(name,OBJPROP_COLOR,theColor);
      if (up)
            ObjectSet(name,OBJPROP_PRICE1,High[i]+gap);
      else  ObjectSet(name,OBJPROP_PRICE1,Low[i] -gap);
}